/****************************************************************************
 This file is a part of Purp (Pugo RPM) - a ncurses-based RPM-handler
 Copyright (C) 1998 - Anders Karlsson <pugo@lysator.liu.se>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         
 
 For latest information and version of Purp, look at the Purp homepage:
 <http://www.lysator.liu.se/purp/>
 
 Please send comments and bug-reports to: <pugo@lysator.liu.se>
****************************************************************************/

#include <sys/stat.h>
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <curses.h>
#include <fcntl.h>

#include "main.h"
#include "linklist.h"
#include "tools.h"
#include "table.h"
#include "dir.h"

#include "progresswin.h"

extern "C" {
      #include <rpm/rpmlib.h>
}

extern struct    tablewin table[];

extern int       panel;
extern char      * rootdir;

extern listelement *listpointer[2];
extern listelement *topListpointer[2];

extern rpmdb grpmdb;  /* for the rpm-database */
extern char *availdir;


//#####################################################################

listelement *ReadInstalledRPM( listelement *lListpointer  )
{
   Header h; 
   int offset;  
   
   table[ INSTALLED ].maxoptions = 0;
   offset = rpmdbFirstRecNum( grpmdb );

   while( offset ) 
     {
	h = rpmdbGetRecord( grpmdb, offset );

	if( ! h ) 
	  {
	     fprintf(stderr, "could not read database record!\n");
	     exit(1);   
	  }
	
	lListpointer = AddItem( lListpointer, h, INSTALLED, "" );
	
	headerFree(h);
	offset = rpmdbNextRecNum( grpmdb, offset );
     }                                    

   return lListpointer;
}

//#####################################################################

listelement *ReadAvailableRPM( listelement *lListpointer )
{
   DIR *theDir;
   struct dirent *lDirent;
   struct stat st;
   int fd;        
   int rc;         
   Header h;
   int isSource;      
   char *name, tmpStr[128];
   int_32 count, type;         
   int dirSize = 0;
   short check;

   /* Check if the user wrote a package-directory */
   if( strcmp( availdir, "" ) )
     {
	theDir = opendir( availdir );
	if(( lDirent = readdir( theDir )))
	  {
	     lstat( lDirent->d_name, &st );
	     
	     while(( lDirent = readdir( theDir )))
	       {
		  dirSize++;
	       }		  
	  }
	
   
	if( dirSize > 0 )
	  {
	     /* Create a progressbar-object */
	     progresswin *progressbar;
	     progressbar = new progresswin( "Reading available packages", dirSize, 0 );
	     
	     
	     rewinddir( theDir );
	     
	     if(( lDirent = readdir( theDir )))
	       {
		  while(( lDirent = readdir( theDir )))
		    {
		       if( IsRPM( lDirent->d_name ) )
			 {
			    sprintf( tmpStr, "%s/%s", availdir, lDirent->d_name );
			    if ((fd = open( tmpStr, O_RDONLY)) >= 0) 
			      {
				 /* Increase the progressbar */
				 progressbar->Update( +1 );
				 
				 fstat( fd, &st );
				 if( ! S_ISDIR( st.st_mode ))
				   {
				      if( ( rpmReadPackageHeader(fd, &h, &isSource, NULL, NULL) ) == 0 )
					{
					   if( h )
					     {
						lListpointer = AddItem( lListpointer, h, AVAILABLE, lDirent->d_name );
						headerFree(h);
					     }
					}
				   }
				 close(fd); 
			      }
			 }
		       
		    }
	       }
	     closedir( theDir );
	     
	     /* Delete the progressbar-object */
	     delete progressbar;
	  }
     }
   return lListpointer;
}
   

/*************************************************************
 * ChangeDir
 * Enter the selected folder..
 *************************************************************/
listelement * ChangeDir( listelement *lListpointer, 
 			 int *doption, 
			 int *top_item,
			 int panel_to_read )
{
   char tmpStr[255];
   
   listelement *choosen = ReturnItemNo( lListpointer, *doption, panel_to_read );
   listelement *tmpPtr = lListpointer;
   
   if( choosen->is_group )
     {
	/* Entering directory */
	lListpointer = (listelement *)choosen->grouplink;

	/* Going UP in hierarchy */
	if( ! strcmp( choosen->name, "..") )
	  {	     
	     *doption = lListpointer->selected_item;
	     *top_item = lListpointer->top_item;

	     /* Check that the currently selected item isn't empty */
	     choosen = ReturnItemNo( lListpointer, *doption, panel_to_read );
	       
	     if( choosen->size == 0 )
		 *doption = GetFirstNotEmpty( lListpointer );
	  }
	else   /* Going DOWN */
	  {	
	     /* Store the item currently selected */
	     tmpPtr->selected_item = *doption;
	     tmpPtr->top_item = *top_item;

     	     /* Going DOWN in hierarchy */
	     tmpPtr = (listelement *) lListpointer;
	     
	     *doption = GetFirstNotEmpty( lListpointer );
	     *top_item = 0;	     
	  }	  
     }
   else
     beep();
   return lListpointer;
}

//#####################################################################

int GetFirstNotEmpty( listelement *lListpointer )
{
   listelement *tmpPtr = lListpointer;
   int count = 0;
   
   /* Go to element after the dummy */
   if( tmpPtr->link != NULL )
     tmpPtr = (listelement *)tmpPtr->link;
   
   while( tmpPtr->is_group && (tmpPtr->size == 0) )
     {
	count++;
	tmpPtr = (listelement *)tmpPtr->link;
     }
   
   return( count );
}



/*************************************************************
 * FindPackageOffset
 * Search the RPM-database and returns the offset for
 * the package specified by the listelement-argument.
 *************************************************************/
int FindPackageOffset( listelement * tmpPtr )
{
   dbiIndexSet matches;
   Header h; 
   int i;
   
   int_32 count, type;
   char * version, * release;

   /* Find all packages that match the name */
   if( rpmdbFindPackage( grpmdb, tmpPtr->name, &matches ) )
     {
	fprintf(stderr, "Error: could not search the RPM-database!\n");
	return( -1 );
     }

   /* Search through matches for packages that also match version and release */
   for( i = 0; i < matches.count; i++ )
     {
	if( matches.recs[ i ].recOffset )
	  {
	     h = rpmdbGetRecord( grpmdb, matches.recs[ i ].recOffset );
	     if( ! h )
	       {
		  fprintf(stderr, "Error: could not read RPM-database record!\n");
		  dbiFreeIndexRecord( matches ); 		       
		  return( -1 );
	       }
	     
	     headerGetEntry( h, RPMTAG_VERSION, &type, (void **) &version, &count ); 
	     headerGetEntry( h, RPMTAG_RELEASE, &type, (void **) &release, &count ); 

	     /* Chech if version and release matches */
	     if( ( ! strcmp( tmpPtr->version, version ) ) && 
		 ( ! strcmp( tmpPtr->release, release ) ) )
	       {
		  /* We have a perfect match! Clean up and return its offset */
		  i = matches.recs[ i ].recOffset;
		  dbiFreeIndexRecord( matches ); 		       
		  headerFree( h );
		  return( i );
	       }
	  }
     }

   /* Package wasn't found, clean up and return.. :-( */
   dbiFreeIndexRecord( matches ); 		       
   headerFree( h );
   return( -2 );
}


int CountEmptyFolders( listelement * lListPointer, int new_option )
{
   int count = 0;
   listelement * tmpPtr = lListPointer;
   
   /* Go to element after the dummy */
   if( tmpPtr->link != NULL )
     tmpPtr = (listelement *)tmpPtr->link;
   
   while( new_option )
     {
	if( tmpPtr->is_group && (tmpPtr->size == 0) )
	  count++;
	new_option--;
	tmpPtr = (listelement *)tmpPtr->link;
     }
   
   return( count );
}

