/****************************************************************************
 This file is a part of Purp (Pugo RPM) - a ncurses-based RPM-handler
 Copyright (C) 1998 - Anders Karlsson <pugo@lysator.liu.se>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         
 
 For latest information and version of Purp, look at the Purp homepage:
 <http://www.lysator.liu.se/purp/>
 
 Please send comments and bug-reports to: <pugo@lysator.liu.se>
****************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <curses.h>

#include "linklist.h"
#include "install.h"
#include "dir.h"
#include "main.h"
#include "table.h"
#include "mainloop.h"
#include "textwindow.h"
#include "tools.h"

#include "cmdwin.h"
#include "progresswin.h"

extern struct tablewin table[];

extern bool      quitloop;
extern int       panel;
extern short     rpmdb_write_access;

extern listelement *topListpointer[2];
extern cmdwin * cmdline;  /* The command-line-object */ 

void MainLoop( listelement *listpointer[2] )
{
   listelement *listPtr;
   int key, count;
   bool result;
   listelement *tmpPtr;
   pwindow *testwin;
//   char tmpstr[64];
   
   /* Put the package-cursor at the top of the panels.. A good place to start.. */
   table[ INSTALLED ].new_option = 0;
   table[ AVAILABLE ].new_option = 0;

   /* Show both panels */
   EraseAll();
   display_menu( listpointer, INSTALLED, table[INSTALLED].new_option );
   display_menu( listpointer, AVAILABLE, table[AVAILABLE].new_option );
   DrawInfoWin();
   cmdline->Draw( MAINLINE );

   /* Main-loop, waiting for commands.. */
   while(quitloop == FALSE)
     {
	noecho();
	keypad(stdscr,TRUE);
	raw();

	key = getch();
	
//	sprintf( tmpstr, "Tecken: %d", key );
//	TermWrite( tmpstr );
	
	switch(key)
	  {
	     /* ^L - Repaint panels */
	   case 12:
             #ifdef DEBUG_TO_TERM
	         TermWrite( "User pressed '^L' (Repaint panels)" );
             #endif

	     EraseAll();
	     display_menu( listpointer, INSTALLED, table[INSTALLED].new_option );
	     display_menu( listpointer, AVAILABLE, table[AVAILABLE].new_option );
	     DrawInfoWin();
	     cmdline->Draw( MAINLINE );
	   break;
	   

	     /* Quit */
	   case 'q':
	   case 'Q':
             #ifdef DEBUG_TO_TERM
	         TermWrite( "User pressed 'Q' (Quit Purp)" );
             #endif
	       HandleQuit();
	   break;
	   

	     /* Pressing enter, entering groups */
	   case 10:
	   case 13:
	   case KEY_ENTER:
	   case KEY_RIGHT:
	   case '6':
             #ifdef DEBUG_TO_TERM
  	        TermWrite( "User pressed 'Enter' (Enter folder)" );
	     #endif
	       HandleEnter( listpointer );
	       break;
	   
	     
	     /* Pressing left key, moving up in hierarchy */
	   case KEY_LEFT:
	   case '4':
             #ifdef DEBUG_TO_TERM
  	        TermWrite( "User pressed 'left' (Move up in hierarchy)" );
	     #endif
	       tmpPtr = listpointer[ panel ]->link;
	       if( ! strcmp( tmpPtr->name, "..") )
		 {
		    table[panel].new_option = 0;
		    HandleEnter( listpointer );
		 }
	       break;

	     
	   /* Up we go.. */  
	   case KEY_UP:
	   case '8':
             #ifdef DEBUG_TO_TERM
	        TermWrite( "User pressed 'Arrow-up' (scroll up)" );
	     #endif
	       HandleKeyUp( listpointer );
	       break;

	     
	   /* Down.. Down.. Down.. */
	   case KEY_DOWN:
	   case '2':
             #ifdef DEBUG_TO_TERM
	        TermWrite( "User pressed 'Arrow-down' (scroll down)" );
	     #endif
	       HandleKeyDown( listpointer );
	     break;

 	     
	   /* Page up.. */
	   case KEY_PPAGE:
             #ifdef DEBUG_TO_TERM
	        TermWrite( "User pressed 'Page-up' (scroll up)" );
	     #endif
	       HandleKeyPageUp( listpointer );
	     break;
	   
	   /* Page down */
	   case KEY_NPAGE:
             #ifdef DEBUG_TO_TERM
	        TermWrite( "User pressed 'Page-down' (scroll down)" );
	     #endif
	       HandleKeyPageDown( listpointer );
	     break;


	   /* KEY_IC = Insert-key, which marks the package */
	   case KEY_IC:
	   case '-':
             #ifdef DEBUG_TO_TERM
	        TermWrite( "User pressed 'Insert' (Mark selection)" );
	     #endif
	       if( rpmdb_write_access )
		 HandleKeyInsert( listpointer );
	     break;
	
	     
	   /* Tab-key, switch panel */
	   case 0x09:
	   case 't':
	   case 'T':
             #ifdef DEBUG_TO_TERM
	        TermWrite( "User pressed 'tab' (Toggle panel)" );
	     #endif
	       HandleKeySwitchPanel( listpointer );
	     break;


	   /* Show description for a package */
	   case 'd':
	   case 'D':
             #ifdef DEBUG_TO_TERM
	        TermWrite( "User pressed 'D' (Describe)" );
             #endif
	       HandleDescribePackage( listpointer );
	     break;

	     
	   /* Show information about Purp */
	   case 'a':
	   case 'A':
             #ifdef DEBUG_TO_TERM
	        TermWrite( "User pressed 'A' (About Purp)" );
             #endif
	       HandleAbout( listpointer );
	     break;
	     
	   /* List.. */
	   case 'f':
	   case 'F':
             #ifdef DEBUG_TO_TERM
	        TermWrite( "User pressed 'f' (follow)" );
	     #endif
	       HandleKeyFollow( listpointer );
	     break;
	     
	   /* List.. */
	   case 'l':
	   case 'L':
             #ifdef DEBUG_TO_TERM
	        TermWrite( "User pressed 'L' (List)" );
	     #endif
	       HandleListPackage( listpointer );
	     break;

	     /* Search.. */
	   case 's':
	   case 'S':
             #ifdef DEBUG_TO_TERM
	        TermWrite( "User pressed 'S' (Search)" );
             #endif
	     HandleKeySearch( listpointer );
	     break;

	   /* Install a package */
	   case 'i':
	   case 'I':
             #ifdef DEBUG_TO_TERM
	        TermWrite( "User pressed 'I' (Install)" );
	     #endif
	       if( rpmdb_write_access )
		 HandleKeyInstall( listpointer );
	     break;

	   /* Upgrade packages */
	   case 'u':
	   case 'U':
             #ifdef DEBUG_TO_TERM
	       TermWrite( "User pressed 'U' (Upgrade)" );
             #endif
	       if( rpmdb_write_access )
		 HandleKeyUpgrade( listpointer );
	     break;

	     
	   /* Kill packages.. */
	   case 'e':
	   case 'E':
             #ifdef DEBUG_TO_TERM
	        TermWrite( "User pressed 'E' (Erase package)" );
	     #endif
	       if( rpmdb_write_access )
		 HandleKeyErase( listpointer );
	     break;

#ifdef DEBUG_TO_TERM
	   case 'x':
	   case 'X':
	     TermWrite( "User pressed 'X' (Dump debug info)" );
	     DumpListTerm( listpointer[ panel ], 0 );
	     break;
#endif
	       
	   default:       
	     break;
	  }
     }
}


/**********************************************************
 * HandleQuit
 * 
 * Quit this marvellous program.. 
 **********************************************************/
void HandleQuit( void )
{
   cmdline->Draw( CONFIRMLINE );
   if( cmdline->HandleConfirmLine() == TRUE )
     {
	clear();	
	quitloop = TRUE;
     }
   else
     {
	cmdline->Draw( MAINLINE );
     }
}


/**********************************************************
 * HandleEnter
 * 
 * Enter the currently selected directory..
 **********************************************************/
void HandleEnter( listelement *listpointer[2] )
{
   listpointer[panel] = ChangeDir( listpointer[ panel ], 
				  &table[ panel ].new_option,
				  &table[ panel ].highest_item_on_list,
				  panel );
   
   werase((panel == 0) ? table[ INSTALLED ].win : 
		                   table[ AVAILABLE ].win );
   
   display_menu(listpointer, panel, table[panel].new_option );
}



/**********************************************************
 * HandleKeyUp
 * 
 * Move the selection-bar up.. Surprising, isn't it?
 **********************************************************/
void HandleKeyUp( listelement *listpointer[2] )
{
   table[panel].new_option = ( table[panel].new_option == 0 ) ? 
                  table[panel].new_option : table[panel].new_option - 1;

   if( table[panel].new_option < table[panel].highest_item_on_list)
     {
	if( table[panel].highest_item_on_list >= (PANEL_CONTENTS_HEIGHT/2) )
	  table[panel].highest_item_on_list -= PANEL_CONTENTS_HEIGHT/2 ;
	else
	  table[panel].highest_item_on_list = 0;
	
	werase( (panel == 0) ? table[ INSTALLED ].win : 
	       table[ AVAILABLE ].win );
     }
   display_menu(listpointer, panel, table[panel].new_option);
}


/**********************************************************
 * HandleKeyDown
 * 
 * Move the selection-bar down..
 **********************************************************/
void HandleKeyDown( listelement *listpointer[2] )
{
   table[panel].new_option = 
     (table[panel].new_option == table[panel].maxoptions - 1) ? 
     table[panel].new_option : table[panel].new_option + 1;
   
   if( table[panel].new_option >= ( table[panel].highest_item_on_list + 
				   PANEL_CONTENTS_HEIGHT))
     {
	if(( table[panel].new_option + PANEL_CONTENTS_HEIGHT / 2) > 
	                                   table[panel].maxoptions )
	  {
	     table[panel].highest_item_on_list = 
	       table[panel].maxoptions - PANEL_CONTENTS_HEIGHT / 2;
	  }
	else
	  {
	     table[panel].highest_item_on_list += PANEL_CONTENTS_HEIGHT / 2 +
	       CountEmptyFolders(listpointer[panel], table[panel].new_option);
	  }
	werase( (panel == 0) ? table[ INSTALLED ].win : 
	       table[ AVAILABLE ].win );
     }
   display_menu(listpointer, panel, table[panel].new_option);
}



/**********************************************************
 * HandleKeyPageUp
 * 
 * Move the selection-bar one page up.. 
 **********************************************************/
void HandleKeyPageUp( listelement *listpointer[2] )
{
   if((table[panel].new_option - PANEL_CONTENTS_HEIGHT) < 0 )
     {
	table[panel].highest_item_on_list = 0;
	table[panel].new_option = 0;
     }
   else
     {
	table[panel].new_option -= PANEL_CONTENTS_HEIGHT;
	table[panel].highest_item_on_list -= PANEL_CONTENTS_HEIGHT;
     }
   werase( (panel == 0) ? table[ INSTALLED ].win : 
	  table[ AVAILABLE ].win );
   display_menu(listpointer, panel, table[panel].new_option);
}



/**********************************************************
 * HandleKeyPageDown
 * 
 * Move the selection-bar one page down..
 **********************************************************/
void HandleKeyPageDown( listelement *listpointer[2] )
{
   if( (table[panel].new_option + PANEL_CONTENTS_HEIGHT ) 
                      >= table[panel].maxoptions )
     {
	if(( table[panel].maxoptions-1) > (table[panel].highest_item_on_list + 
					   PANEL_CONTENTS_HEIGHT))
	  {
	     table[panel].highest_item_on_list += PANEL_CONTENTS_HEIGHT;
	  }
	table[panel].new_option = table[panel].maxoptions - 1;	       
     }
   else
     {
	table[panel].new_option += PANEL_CONTENTS_HEIGHT;
	table[panel].highest_item_on_list += PANEL_CONTENTS_HEIGHT;
     }
 
   werase( (panel == 0) ? table[ INSTALLED ].win : 
	                  table[ AVAILABLE ].win );
   
   display_menu(listpointer, panel, table[panel].new_option);
}



/**********************************************************
 * HandleKeyInsert
 * 
 * Toggle the mark-flag for the currently selected
 * package..
 **********************************************************/
void HandleKeyInsert( listelement *listpointer[2] )
{
   if( ToggleMark( listpointer[panel], table[panel].new_option, panel ) )
     {
	table[panel].new_option = 
	  (table[panel].new_option == table[panel].maxoptions-1) ? 
	  table[panel].new_option : table[panel].new_option + 1;
	
	if( table[panel].new_option >= ( table[panel].highest_item_on_list + 
					PANEL_CONTENTS_HEIGHT))
	  {
	     table[panel].highest_item_on_list++;
	     werase( (panel == 0) ? table[ INSTALLED ].win : 
		    table[ AVAILABLE ].win );
	  }
	
	display_menu(listpointer, panel, table[panel].new_option);
     }
}



/**********************************************************
 * HandleDescribePackage
 * 
 * Show description for the currently selected package..
 **********************************************************/
void HandleDescribePackage( listelement *listpointer[2] )
{
   char tmpstr[128];
   listelement *tmpPtr;

   tmpPtr = ReturnItemNo( listpointer[ panel ], table[panel].new_option, panel );
   
   if( ! tmpPtr->is_group )
     {
	sprintf( tmpstr, "Info for: %s-%s-%s", tmpPtr->name, tmpPtr->version, tmpPtr->release );
	
	textwin *describewin;
	describewin = new textwin( tmpstr, 
				  "Press any key to continue..", 
				  tmpPtr->description, 0, WRAP );
	
	getch();
	delete describewin;
	
	display_menu( listpointer, 0, table[ INSTALLED ].new_option );
	display_menu( listpointer, 1, table[ AVAILABLE ].new_option );
	DrawInfoWin();
	cmdline->Draw( MAINLINE );
     }
}



/**********************************************************
 * HandleAbout
 * 
 * Show information about Purp
 **********************************************************/
void HandleAbout( listelement *listpointer[2] )
{
   textwin *aboutWin;
   aboutWin = new textwin( "About Purp", 
			   "Press any key to continue..", 
			   "", 0, WRAP );
	
   aboutWin->AddStr( " PURP (Pugo RPM) Version 0.3.0 - A ncurses-based RPM-handler \n", NOWRAP );
   aboutWin->AddStr( " Copyright (C) 1998 - Anders Karlsson <pugo@lysator.liu.se>\n\n", NOWRAP );

   aboutWin->AddStr( " This program is free software; you can redistribute it and/or modify\n", NOWRAP );
   aboutWin->AddStr( " it under the terms of the GNU General Public License as published by\n", NOWRAP );
   aboutWin->AddStr( " the Free Software Foundation; either version 2 of the License, or\n", NOWRAP );
   aboutWin->AddStr( " (at your option) any later version.\n\n", NOWRAP );

   aboutWin->AddStr( " This program is distributed in the hope that it will be useful,\n", NOWRAP );
   aboutWin->AddStr( " but WITHOUT ANY WARRANTY; without even the implied warranty of\n", NOWRAP );
   aboutWin->AddStr( " MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n", NOWRAP );
   aboutWin->AddStr( " GNU General Public License for more details.\n\n", NOWRAP );

   aboutWin->AddStr( " You should have received a copy of the GNU General Public License\n", NOWRAP );
   aboutWin->AddStr( " along with this program; if not, write to the Free Software\n", NOWRAP );
   aboutWin->AddStr( " Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n", NOWRAP );

   aboutWin->AddStr( " For latest information and version of Purp, look at the Purp homepage:\n", NOWRAP );
   aboutWin->AddStr( " <http://www.lysator.liu.se/purp/>\n\n", NOWRAP );

   aboutWin->AddStr( " Please send comments and bug-reports to: <pugo@lysator.liu.se>\n", NOWRAP );
   
   getch();
   delete aboutWin;
	
   display_menu( listpointer, 0, table[ INSTALLED ].new_option );
   display_menu( listpointer, 1, table[ AVAILABLE ].new_option );
   DrawInfoWin();
   cmdline->Draw( MAINLINE );

}


/**********************************************************
 * HandleListPackage
 * 
 * Show info about the currently selected package..
 **********************************************************/
void HandleListPackage( listelement *listpointer[2] )
{
   listelement *tmpPtr;

   tmpPtr = ReturnItemNo( listpointer[ panel ], table[panel].new_option, panel );
   
   if( ! tmpPtr->is_group )
     {
	cmdline->Draw( LISTLINE );
	cmdline->HandleListLine( listpointer[panel], panel, table[panel].new_option );
	
	display_menu( listpointer, INSTALLED, table[INSTALLED].new_option );
	display_menu( listpointer, AVAILABLE, table[AVAILABLE].new_option );
	DrawInfoWin();
	cmdline->Draw( MAINLINE );
     }
}

/**********************************************************
 * HandleKeyFollow
 * 
 * Moves the panel not active so it shows the same
 * folder as the selected (if possible).
 **********************************************************/
void HandleKeyFollow( listelement *listpointer[2] )
{
   char tmpStr[256] = ""; 
   char *strPtr; 
   char *groupname;
   short found_folder = 1;
   int   other_panel = (panel == 1 ? 0 : 1);

   listelement *selectedPtr, *groupPtr;;
   listelement *listPtr = topListpointer[ other_panel ];
   
   /* Copy group-path */
   strcpy( tmpStr, listpointer[panel]->group );
   
   
   /* Initiate strtok, get first path-name */
   groupname = strtok( tmpStr, "/" );
   
   /* Traverse down in hierarchy and search for folder */
   while( groupname != NULL )
     {
	/* Break if the group doesn't exist */
	if( (groupPtr = FindGroup( listPtr, groupname )) == NULL )
	  {
	     found_folder = 0;
	     break;
	  }
	
	/* Enter the group */
	listPtr = (listelement *)groupPtr->grouplink;
	
	groupname = strtok( NULL, "/" );
     }

   
   /* Go to wanted folder if it did exist */
   if( found_folder )
     {
	listpointer[ other_panel ] = listPtr;
	table[ other_panel ].new_option = 0;
	table[ other_panel ].highest_item_on_list = 0;
	
	/* Show results */
	werase( table[ other_panel ].win );
	display_menu(listpointer, other_panel, table[ other_panel ].new_option );
     }
}


/**********************************************************
 * HandleKeySearch
 * 
 * Search for a package
 **********************************************************/
void HandleKeySearch( listelement *listpointer[2] )
{
   listelement * found_package;
   char name[256];
   int count = 0;
   
   cmdline->Draw( SEARCHLINE );
   cmdline->HandleSearchLine( panel, name );

   found_package = CheckPackageExists( panel, name, NULL, NULL, CASE_INSENSITIVE );
   
   
   if( found_package->name )
     {
	/* Go up to dummy */
	while( ! found_package->is_dummy )
	  {
	     count++;
	     found_package = (listelement *)found_package->prevlink;
	  }
	
	listpointer[panel] = found_package;
	table[panel].new_option = count - 1;

	/* Highest item on the list */
	if( count <= PANEL_CONTENTS_HEIGHT )
	  table[panel].highest_item_on_list = 0;
	else
	  table[panel].highest_item_on_list = count - PANEL_CONTENTS_HEIGHT/2;
	
	
	/* Show results */
	werase((panel == 0) ? table[ INSTALLED ].win : 
	       table[ AVAILABLE ].win );
	display_menu(listpointer, panel, table[panel].new_option );
     }
   DrawInfoWin();
   
   cmdline->Draw( MAINLINE );
}



/**********************************************************
 * HandleKeySwitchPanel
 * 
 * Toggles selected panel
 **********************************************************/
void HandleKeySwitchPanel( listelement *listpointer[2] )
{
   char tmpStr[255];

   int tmp_panel = (panel == 0) ? 1 : 0;
   listelement *tmpPtr = topListpointer[ tmp_panel ];
   
   if( tmpPtr->straightlink != NULL )
     {
	panel = tmp_panel;
	display_menu(listpointer, 0, table[panel].new_option);
	display_menu(listpointer, 1, table[panel].new_option);
     }
}



/**********************************************************
 * HandleKeyInstall
 * 
 * Handles install-events..
 **********************************************************/
void HandleKeyInstall( listelement *listpointer[2] )
{
   listelement *tmpPtr;
   short installTemporary = 0;

   if( topListpointer[AVAILABLE]->straightlink != NULL )
     {
	if( table[AVAILABLE].selected == 0 )
	  {
	     tmpPtr = ReturnItemNo( listpointer[ AVAILABLE ], 
				   table[ AVAILABLE ].new_option, 
				   panel );
	     
	     if( ! tmpPtr->is_group )
	       {
		  tmpPtr->is_marked = TRUE;
		  InstallPackages( INSTALL, INSTALL_TEMPORARY );
	       }
	  }
	else
	  InstallPackages( INSTALL, INSTALL_MARKED );
	
	cmdline->Draw( MAINLINE );
	ClearPanel( AVAILABLE );
	display_menu( listpointer, INSTALLED, table[ INSTALLED ].new_option );
	display_menu( listpointer, AVAILABLE, table[ AVAILABLE ].new_option );
	DrawInfoWin();
     }
}
   
/**********************************************************
 * HandleKeyUpgrade
 * 
 * Handles upgrade-events..
 **********************************************************/
void HandleKeyUpgrade( listelement *listpointer[2] )
{
   listelement *tmpPtr;
   short installTemporary = 0;

   if( topListpointer[AVAILABLE]->straightlink != NULL )
     {
	if( table[AVAILABLE].selected == 0 )
	  {
	     tmpPtr = ReturnItemNo( listpointer[ AVAILABLE ], 
				   table[ AVAILABLE ].new_option, 
				   panel );
	     
	     if( ! tmpPtr->is_group )
	       {
		  tmpPtr->is_marked = TRUE;
		  InstallPackages( UPGRADE, INSTALL_TEMPORARY );
	       }
	  }
	else
	  InstallPackages( UPGRADE, INSTALL_MARKED );
   
	cmdline->Draw( MAINLINE );
	ClearPanel( INSTALLED );
	ClearPanel( AVAILABLE );
	display_menu( listpointer, INSTALLED, table[ INSTALLED ].new_option );
	display_menu( listpointer, AVAILABLE, table[ AVAILABLE ].new_option );
	DrawInfoWin();
     }
}


/**********************************************************
 * HandleKeyErase
 * 
 * Handles erase-events..
 **********************************************************/
void HandleKeyErase( listelement *listpointer[2] )
{
   listelement *tmpPtr;
   short installTemporary = 0;
   
   if( table[ INSTALLED ].selected == 0 )
     {
	tmpPtr = ReturnItemNo( listpointer[ INSTALLED ], 
			       table[ INSTALLED ].new_option, 
			       panel );

	if( ! tmpPtr->is_group )
	  {
	     tmpPtr->is_marked = TRUE;
	     InstallPackages( UNINSTALL, INSTALL_TEMPORARY );
	  }
     }
   else
     InstallPackages( UNINSTALL, INSTALL_MARKED );

   cmdline->Draw( MAINLINE );
   ClearPanel( INSTALLED );
   ClearPanel( AVAILABLE );
   display_menu( listpointer, INSTALLED, table[ INSTALLED ].new_option );
   display_menu( listpointer, AVAILABLE, table[ AVAILABLE ].new_option );
   DrawInfoWin();
}
