/****************************************************************************
 This file is a part of Purp (Pugo RPM) - a ncurses-based RPM-handler
 Copyright (C) 1998 - Anders Karlsson <pugo@lysator.liu.se>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         
 
 For latest information and version of Purp, look at the Purp homepage:
 <http://www.lysator.liu.se/purp/>
 
 Please send comments and bug-reports to: <pugo@lysator.liu.se>
****************************************************************************/
  
#include <curses.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

#include "main.h"
#include "linklist.h"
#include "mainloop.h"
#include "table.h"
#include "cmdwin.h"
#include "pathwin.h"
#include "tools.h"
#include "dir.h"

extern struct tablewin table[];

extern WINDOW  * infowin;
extern int       panel;
extern cmdwin  * cmdline;
extern pathwin * pathline; 

//#####################################################################

void InitieraCurs()
{
   if( !initTest( &table[ INSTALLED ].win, &table[ AVAILABLE ].win, &infowin ) ) exit(1);

     if( has_colors() )
     {
	/* Set color-pairs */
	init_pair( 1, COLOR_WHITE, COLOR_BLACK );
	init_pair( 2, COLOR_WHITE, COLOR_BLUE );
	init_pair( 3, COLOR_GREEN, COLOR_BLACK );
	init_pair( 4, COLOR_RED, COLOR_BLACK );
	init_pair( 5, COLOR_GREEN, COLOR_RED );
	init_pair( 6, COLOR_GREEN, COLOR_BLUE );
	init_pair( 10, COLOR_YELLOW, COLOR_BLACK );
	wattrset( table[ INSTALLED ].win, COLOR_PAIR( 2 ) );
	wattrset( table[ AVAILABLE ].win, COLOR_PAIR( 2 ) );
	wattrset( infowin, COLOR_PAIR( 2 ) );
     }

   werase( table[ INSTALLED ].win );   
   werase( table[ AVAILABLE ].win );
   werase( infowin );
    
   wbkgd( table[ INSTALLED ].win, COLOR_PAIR( 2 ) );
   wbkgd( table[ AVAILABLE ].win, COLOR_PAIR( 2 ) );
   wbkgd( infowin, COLOR_PAIR( 2 ) );

   PrepareInstalledWin();
   PrepareAvailableWin();
   DrawInfoWin();
}


//#####################################################################
void DrawInfoWin()
{
   wborder( infowin, 0, 0, 0, 0, ACS_LTEE, ACS_RTEE, 0, 0);

   mvwaddstr( infowin, 1, 2, "Name:        ");
   mvwaddstr( infowin, 2, 2, "Version:     ");
   mvwaddstr( infowin, 3, 2, "Release:     ");
   mvwaddstr( infowin, 5, 2, "Summary:     ");

   mvwaddstr( infowin, 1, 40, "Size:        ");
   mvwaddstr( infowin, 2, 40, "Installdate: ");
   mvwaddstr( infowin, 3, 40, "Builddate: ");

   wrefresh( infowin );
}


void PrepareInstalledWin()
{
   wborder( table[ INSTALLED ].win, 0, 0, 0, 0, 0, 0, ACS_LTEE, 0);

   mvwaddch( table[ INSTALLED ].win, 0, 2, ACS_RTEE );
   mvwaddstr( table[ INSTALLED ].win, 0, 3, " Installed packages ");   
   mvwaddch( table[ INSTALLED ].win, 0, 23, ACS_LTEE );

   wrefresh( table[ INSTALLED ].win );
}

void PrepareAvailableWin()
{
   wborder( table[ AVAILABLE ].win, 0, 0, 0, 0, 0, 0, 0, ACS_RTEE);

   mvwaddch( table[ AVAILABLE ].win, 0, 2, ACS_RTEE );
   mvwaddstr( table[ AVAILABLE ].win, 0, 3, " Available packages ");   
   mvwaddch( table[ AVAILABLE ].win, 0, 23, ACS_LTEE );

   wrefresh( table[ AVAILABLE ].win );
}

//#####################################################################

int initTest (WINDOW **lWin1, WINDOW **lWin2, WINDOW **lInfoWin )
{
   
   initscr();
   
   if (has_colors())
     start_color();

   table[ INSTALLED ].width  = COLS/2;
   table[ AVAILABLE ].width  = COLS/2;
   table[ INSTALLED ].height = LINES;            
   table[ AVAILABLE ].height = LINES;            

   *lWin1 = newwin( PANEL_HEIGHT, table[ INSTALLED ].width, 0, 0);
   
   *lWin2 = newwin( PANEL_HEIGHT, table[ AVAILABLE ].width, 0, 
		   table[ AVAILABLE ].width);


   *lInfoWin = newwin( INFO_WIN_HEIGHT, COLS, INFO_WIN_TOP, 0);

   if( *lWin1 == NULL )
    {   
       fprintf( stderr, "Error: Unable to create installed-window!\n" );
       endwin();
       return 0;
    }
   
   if( *lWin2 == NULL )
    {   
       fprintf( stderr, "Error: Unable to create available-window!\n" );
       endwin();
       return 0;
    }
   
   if( *lInfoWin == NULL )
    {   
       fprintf( stderr, "Error: Unable to create info-window!\n" );
       endwin();
       return 0;
    }

return 1;
}

//#####################################################################

void display_menu(listelement *lListpointer[2],
	     int panel_to_show,
	     int new_option)
{
   int i=0, j=0, xpos = 1, empty_folders;
   listelement *localListPtr, *localListPtr2, *tmpPtr;
   WINDOW *current_win;
   char tmpstr[255];
   char *empty_string = "                                                                                ";
   
   localListPtr = lListpointer[ panel_to_show ];
   localListPtr2 = lListpointer[ panel_to_show ];

   current_win = (panel_to_show == 0 ) ? table[ INSTALLED ].win : 
                                         table[ AVAILABLE ].win;


   /*
    * Count the number of elements on the list 
    */
   
   if( localListPtr->link != NULL )
     localListPtr = (listelement *)localListPtr->link;
   
   table[ panel_to_show ].maxoptions = 0;
   while( localListPtr != NULL )
     {
	table[ panel_to_show ].maxoptions++;
	localListPtr = (listelement *)localListPtr->link;
     }
   
	localListPtr = lListpointer[ panel_to_show ];
	
	/* Jump to the element after the dummy.. */
	if( localListPtr != NULL ) localListPtr = (listelement *)localListPtr -> link;
	
	
	/* Move to the first item we want to show... */
	j = 0;
	while( j < table[ panel_to_show ].highest_item_on_list )
	  {
	     localListPtr = (listelement *)localListPtr -> link;
	     j++;
	  }
	
	j = 1;
	
	
	/* 
	 * Fill the panel with packets
	 */
	
	while( (localListPtr != NULL) && (j <= PANEL_CONTENTS_HEIGHT))
	  {     
	     /* Calculate the package-names */
	     if( localListPtr->is_group )
	       sprintf( tmpstr, "%s/\n", localListPtr->name );
	     else
	       sprintf( tmpstr, "%s%s-%s-%s\n", 
		       (localListPtr->is_marked == TRUE)?"*":" ",
		       localListPtr->name, 
		       localListPtr->version, 
		       localListPtr->release );
	
	     
	     if( localListPtr->is_marked )
	       {
		  wattrset( current_win, A_BOLD ); 
		  wattrset( current_win, COLOR_PAIR(6) ); 
	       }
	     else
	       wattrset( current_win, COLOR_PAIR(2) ); 
	     
	     mvwaddstr( current_win, j + PANEL_TOP, 1, tmpstr );
	     j = j + 1;
	     
	     localListPtr = (listelement *)localListPtr -> link;
	  }

   /*
    * Draw the selection-bar 
    */
   
   wattrset( current_win, A_REVERSE);
   
   if( panel_to_show == panel )
     {
	tmpPtr = ReturnItemNo( localListPtr2, new_option, panel_to_show );
	
	if( tmpPtr->is_group )
	  sprintf( tmpstr, "%s/\n", tmpPtr->name );
	else
	  sprintf( tmpstr, "%s%s-%s-%s\n", 
		  (tmpPtr->is_marked == TRUE)?"*":" ",
		  tmpPtr->name, 
		  tmpPtr->version, 
		  tmpPtr->release );
	
	mvwaddstr( current_win, PANEL_TOP + new_option - 
		  table[ panel_to_show ].highest_item_on_list + 1,
		  1, tmpstr);
	
	/* Update path-window */
	if( tmpPtr->is_group )
	  sprintf( tmpstr, "%s", tmpPtr->group );
	else
	  sprintf( tmpstr, "/%s/%s-%s-%s", tmpPtr->group,
		  tmpPtr->name,
		  tmpPtr->version,
		  tmpPtr->release );
	
	pathline->PrintPath( tmpstr );
	
	/*
	 * Update the file-info-window
	 */
	
	mvwaddnstr( infowin, 1, 15, empty_string, 24 );
	
	mvwaddnstr( infowin, 1, 15, tmpPtr->name, 24 );
	
	mvwaddnstr( infowin, 2, 15, empty_string, 16 );
	mvwaddnstr( infowin, 3, 15, empty_string, 10 );
	mvwaddnstr( infowin, 5, 15, empty_string, 60 );
	mvwaddnstr( infowin, 1, 48, empty_string, 25 );
	mvwaddnstr( infowin, 2, 53, empty_string, 25 );
	mvwaddnstr( infowin, 3, 53, empty_string, 25 );
	
	
	if( ! tmpPtr->is_group )
	  {
	     /* Print name, version, release */
	     mvwaddnstr( infowin, 2, 15, tmpPtr->version, 16 );
	     mvwaddnstr( infowin, 3, 15, tmpPtr->release, 10 );
	     
	     /* Print summary */
	     if( strlen( tmpPtr->summary ) > 0 )
	       mvwaddnstr( infowin, 5, 15, tmpPtr->summary, 60 );
	     
	     /* Print size */
	     sprintf(tmpstr, "%d", tmpPtr->size);
	     mvwaddnstr( infowin, 1, 53, tmpstr, 20 );
	     
	     /* Print either installdate or builddate */
	     if( panel_to_show == INSTALLED )
	       {
		  mvwaddnstr( infowin, 2, 53, ctime( &tmpPtr->installdate ), 24 );
		  mvwaddnstr( infowin, 3, 53, ctime( &tmpPtr->builddate ), 24 );
	       }
	     else
	       {
		  mvwaddnstr( infowin, 3, 53, ctime( &tmpPtr->builddate ), 24 );
	       }
	  }
	wrefresh( infowin );
     }
   
   
   /*
    * Draw borders and dividers 
    */
   
   wattrset( current_win, COLOR_PAIR(2)  );   
   
   if( panel_to_show == INSTALLED )
     {
	wborder(current_win, 0, 0, 0, 0, 0, 0, ACS_LTEE, 0);
	mvwaddstr( current_win, 0, 3, " Installed packages " );   
     }
   else
     {
	wborder(current_win, 0, 0, 0, 0, 0, 0, 0, ACS_RTEE);
	mvwaddstr( current_win, 0, 3, " Available packages " );   
     }
   
   /* Draw panel-title */
   mvwaddch( current_win, 0, 2, ACS_RTEE );
   mvwaddch( current_win, 0, 23, ACS_LTEE );
   
   
   /* Draw # selected packages and total selected size */
   sprintf( tmpstr, " (%d / %d KB) ", table[ panel_to_show ].selected, 
	   table[ panel_to_show ].selected_size / 1024 );
   
   mvwaddstr( current_win, PANEL_HEIGHT - 1, 
	     table[ panel_to_show ].width - strlen(tmpstr) - 2, tmpstr );
   
   wrefresh( current_win );
}

//#####################################################################

void ClearPanel( short panel )
{
   WINDOW *theWin;

   theWin = ( panel == 0 ) ? table[ INSTALLED ].win : 
                             table[ AVAILABLE ].win;
   
   werase( theWin );   
   wrefresh( theWin );
}


//#####################################################################

void EraseAll()
{
   erase();
   refresh();
}



//#####################################################################

void MyPutText( int x, int y, char *text )
{
   mvaddstr( y, x, text );
   refresh();
}


//#####################################################################


